AVAIABLE_RIGS = {
    'Standup L-Set Advanced Component Manufacturing Efficiency I',
    'Standup L-Set Advanced Large Ship Manufacturing Efficiency I',
    'Standup L-Set Advanced Medium Ship Manufacturing Efficiency I',
    'Standup L-Set Advanced Small Ship Manufacturing Efficiency I',
    'Standup L-Set Ammunition Manufacturing Efficiency I',
    'Standup L-Set Basic Capital Component Manufacturing Efficiency I',
    'Standup L-Set Basic Large Ship Manufacturing Efficiency I',
    'Standup L-Set Basic Medium Ship Manufacturing Efficiency I',
    'Standup L-Set Basic Small Ship Manufacturing Efficiency I',
    'Standup L-Set Capital Ship Manufacturing Efficiency I',
    'Standup L-Set Drone and Fighter Manufacturing Efficiency I',
    'Standup L-Set Equipment Manufacturing Efficiency I',
    'Standup L-Set Structure Manufacturing Efficiency I',
    'Standup L-Set Thukker Advanced Component Manufacturing Efficiency',
    'Standup L-Set Thukker Basic Capital Component Manufacturing Efficiency',
    'Standup L-Set Advanced Component Manufacturing Efficiency II',
    'Standup L-Set Advanced Large Ship Manufacturing Efficiency II',
    'Standup L-Set Advanced Medium Ship Manufacturing Efficiency II',
    'Standup L-Set Advanced Small Ship Manufacturing Efficiency II',
    'Standup L-Set Capital Ship Manufacturing Efficiency II',
    'Standup L-Set Ammunition Manufacturing Efficiency II',
    'Standup L-Set Basic Capital Component Manufacturing Efficiency II',
    'Standup L-Set Basic Large Ship Manufacturing Efficiency II',
    'Standup L-Set Basic Medium Ship Manufacturing Efficiency II',
    'Standup L-Set Basic Small Ship Manufacturing Efficiency II',
    'Standup L-Set Drone and Fighter Manufacturing Efficiency II',
    'Standup L-Set Equipment Manufacturing Efficiency II',
    'Standup L-Set Structure Manufacturing Efficiency II',
}



import json

from django.shortcuts import render
from django.http import FileResponse
from django.contrib.auth.decorators import login_required
from django.contrib import messages
from django.core.files.base import ContentFile

from allianceauth.services.hooks import get_extension_logger

from .forms import ExportForm

logger = get_extension_logger(__name__)


@login_required
def index(request):
    if request.method == 'POST':
        form = ExportForm(request.user, request.POST, request.FILES)
        if form.is_valid():
            error = False
            config = json.load(form.cleaned_data['config'])
            ownership = (
                request.user.character_ownerships
                .select_related('character')
                .get(character__character_id=form.cleaned_data['character'])
            )

            if form.cleaned_data['skills'] == 'memberaudit':
                from .exporters.skills.memberaudit import import_skills, is_character_added

                if not is_character_added(ownership.character):
                    messages.error(request, f"Character {ownership.character.character_name} is not added to MemberAudit")
                    error = True
                else:
                    config.update(import_skills(ownership.character))
            elif form.cleaned_data['skills'] == 'corptools':
                from .exporters.skills.corptools import import_skills, is_character_added

                if not is_character_added(ownership.character):
                    messages.error(request, f"Character {ownership.character.character_name} is not added to CorpTools")
                    error = True
                else:
                    config.update(import_skills(ownership.character))

            if form.cleaned_data['structures']:
                from .exporters.structures.structures import export_structures

                config['hidden_my_structures'] = export_structures(ownership.user)

            if not error:
                updated_config = ContentFile(json.dumps(config, indent=4).encode())
                return FileResponse(updated_config, as_attachment=True, filename='config.json')

    else:
        form = ExportForm(request.user)

    context = {
        'form': form,
    }

    return render(request, 'ravworks_exporter/index.html', context=context)



from allianceauth.eveonline.models import EveCharacter

from corptools.models import Skill, CharacterAudit

from .skill_settings import SKILL_SET


def import_skills(character: EveCharacter):
    skills = Skill.objects.filter(
        character__character=character,
        skill_id__in=SKILL_SET,
    )

    res = {f"skill_{skill.skill_id}": skill.active_skill_level if skill.active_skill_level != 0 else 1 for skill in skills}

    missing_skills = SKILL_SET.difference(skills.values_list('skill_id', flat=True))
    res.update({f"skill_{skill_id}": 1 for skill_id in missing_skills})

    return res


def is_character_added(character: EveCharacter):
    return CharacterAudit.objects.filter(character=character).exists()



from django import forms

from django.conf import settings


class ExportForm(forms.Form):
    config = forms.FileField(required=True, label='Config file')
    skills = forms.ChoiceField(required=True, label='Skills', choices=[
        (None, 'Do not export'),
    ])
    character = forms.ChoiceField(required=True, label='Character')
    structures = forms.BooleanField(initial=True, required=False, label='Structures')

    def __init__(self, user, *args, **kwargs):
        super().__init__(*args, **kwargs)

        if 'structures' not in settings.INSTALLED_APPS:
            self.fields['structures'].disabled = True
            self.fields['structures'].initial = False
            self.fields['structures'].label = 'Structures (not available)'

        if 'memberaudit' in settings.INSTALLED_APPS and user.has_perm('memberaudit.basic_access'):
            self.fields['skills'].choices = [*self.fields['skills'].choices, ('memberaudit', 'MemberAudit')]

        if 'corptools' in settings.INSTALLED_APPS:
            self.fields['skills'].choices = [*self.fields['skills'].choices, ('corptools', 'CorpTools')]

        self.fields['skills'].initial = self.fields['skills'].choices[-1][0]

        self.fields['character'].choices = [
            (ownership.character.character_id, ownership.character.character_name)
            for ownership in user.character_ownerships.select_related('character').all()
        ]



from structures.models import Structure
from structures.constants import EveCategoryId

from .structures_settings import AVAIABLE_RIGS

FIT_PERMISSIONS = ["structures.view_structure_fit"]


def extract_slot_assets(fittings: list, slot_name: str) -> list:
    """Return assets for slot sorted by slot number"""
    return [
        asset[0]
        for asset in sorted(
            [
                (asset, asset.location_flag[-1])
                for asset in fittings
                if asset.location_flag.startswith(slot_name)
            ],
            key=lambda x: x[1],
        )
    ]


def export_structures(user):
    res = []
    if user.has_perms(FIT_PERMISSIONS):
        structures = (
            Structure.objects
            .visible_for_user(user)
            .filter(eve_type__eve_group__eve_category_id=EveCategoryId.STRUCTURE)
            .select_related("eve_solar_system", 'eve_type')
            .prefetch_related('items__eve_type')
        )

        for i, structure in enumerate(structures, 1):
            assets = structure.items.all()
            rig_slots = extract_slot_assets(assets, "RigSlot")

            rigs = {f"Rig{j}": "No Rig" for j in range(1, 4)}

            for j, rig in enumerate(rig_slots, 1):
                if rig.eve_type.name in AVAIABLE_RIGS:
                    rigs[f"Rig{j}"] = rig.eve_type.name

            if structure.eve_solar_system.is_high_sec:
                security = 'High'
            elif structure.eve_solar_system.is_low_sec:
                security = 'Low'
            else:
                security = 'Null / Wormhole'

            res.append({
                'id': f"Structure {i}",
                'name': structure.name,
                'security': security,
                'structure': structure.eve_type.name,
                **rigs,
            })

    return res
